#!/bin/sh
# Ensure ln handles non-UTF-8 source names in target-dir mode.

# Copyright (C) 2026 Free Software Foundation, Inc.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

. "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
print_ver_ ln

echo a > "$(bad_unicode)" \
  || skip_ 'bad unicode not supported in shell or file system'

name="$(bad_unicode)"

# Check source file and create target directory.
test -f "$name" || framework_failure_
mkdir dst || framework_failure_

# Hard link in target directory.
ln "$name" dst || fail=1
test -f "dst/$name" || fail=1
rm -f "dst/$name" || framework_failure_

# Symbolic link with -t.
ln -s -t dst "$name" || fail=1
test -L "dst/$name" || fail=1

Exit $fail
